package hu.bme.mit.jdt.info.handlers;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.handlers.HandlerUtil;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IMethod;
import org.eclipse.jdt.core.IPackageFragment;
import org.eclipse.jdt.core.IPackageFragmentRoot;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.text.Document;

/**
 * Our sample handler extends AbstractHandler, an IHandler base class.
 * 
 * @see org.eclipse.core.commands.IHandler
 * @see org.eclipse.core.commands.AbstractHandler
 */
public class JDTInfoHandler extends AbstractHandler {
	/**
	 * The constructor.
	 */
	public JDTInfoHandler() {
	}

	/**
	 * the command has been executed, so extract extract the needed information
	 * from the application context.
	 */
	public Object execute(ExecutionEvent event) throws ExecutionException {
		IWorkspace workspace = ResourcesPlugin.getWorkspace();
		IWorkspaceRoot root = workspace.getRoot();
		// Get all projects in the workspace
		IProject[] projects = root.getProjects();
		for (IProject project : projects) {
			try {
				// Output various information about source projects
				printProjectInfo(project);
				// Create package
				createJavaPackage(project);
			} catch (CoreException e) {
				e.printStackTrace();
			}
		}
		return null;
	}

	private void createJavaPackage(IProject project) throws CoreException {
		if (project.isNatureEnabled("org.eclipse.jdt.core.javanature")) {
			IJavaProject javaProject = JavaCore.create(project);
			IFolder folder = project.getFolder("src");
			IPackageFragmentRoot srcFolder = javaProject
					.getPackageFragmentRoot(folder);
			IPackageFragment fragment = srcFolder.createPackageFragment(project
					.getName() + ".created", true, null);
		}

	}

	private void printProjectInfo(IProject project) throws CoreException {
		System.out.println("Working in project " + project.getName());
		// Check for Java project
		if (project.isNatureEnabled("org.eclipse.jdt.core.javanature")) {
			IJavaProject javaProject = JavaCore.create(project);
			printPackageInfo(javaProject);
		}
	}

	private void printPackageInfo(IJavaProject project)
			throws JavaModelException {
		IPackageFragment[] packages = project.getPackageFragments();
		for (IPackageFragment javaPackage : packages) {
			// Package fragments include all packages in the classpath
			// We will only look at the package from the source folder
			// K_BINARY would also include JARS, e.g. rt.jar
			if (javaPackage.getKind() == IPackageFragmentRoot.K_SOURCE) {
				System.out.println("Package " + javaPackage.getElementName());
				printCompilationUnitInfo(javaPackage);
			}
		}
	}

	private void printCompilationUnitInfo(IPackageFragment javaPackage)
			throws JavaModelException {
		for (ICompilationUnit unit : javaPackage.getCompilationUnits()) {
			System.out.print("Source file " + unit.getElementName() + " ");
			Document doc = new Document(unit.getSource());
			System.out.println("# of lines: " + doc.getNumberOfLines());
			printIMethods(unit);
		}
	}

	private void printIMethods(ICompilationUnit unit) throws JavaModelException {
		IType[] allTypes = unit.getAllTypes();
		for (IType type : allTypes) {
			for (IMethod method : type.getMethods()) {
				System.out.println("Method name: " + method.getElementName());
				System.out
						.println("Method signature: " + method.getSignature());
				System.out.println("Return type: " + method.getReturnType());
			}
		}

	}
}
